@@ -104,14 +104,13 @@ KDateTable::KDateTable(const QDate &date, QWidget *parent)
 
 KDateTable::KDateTable(QWidget *parent)
     : QWidget(parent)
-    , d(new KDateTablePrivate(this))
+    , d(std::make_unique<KDateTablePrivate>(this))
 {
     initWidget(QDate::currentDate());
 }
 
 KDateTable::~KDateTable()
 {
-    delete d;
 }
 
 void KDateTable::initWidget(const QDate &date)
@@ -298,7 +297,7 @@ void KDateTable::paintCell(QPainter *painter, int row, int col)
 
         // Draw the day number in the cell, if the date is not valid then we don't want to show it
         if (validDay) {
-            cellText = QString::number(cellDate.day());
+            cellText = locale().toString(cellDate.day());
         } else {
             cellText = QString();
         }
@@ -532,12 +531,12 @@ bool KDateTable::event(QEvent *ev)
     switch (ev->type()) {
     case QEvent::HoverMove: {
         QHoverEvent *e = static_cast<QHoverEvent *>(ev);
-        const int row = e->pos().y() * d->m_numWeekRows / height();
+        const int row = e->position().y() * d->m_numWeekRows / height();
         int col;
         if (layoutDirection() == Qt::RightToLeft) {
-            col = d->m_numDayColumns - (e->pos().x() * d->m_numDayColumns / width()) - 1;
+            col = d->m_numDayColumns - (e->position().x() * d->m_numDayColumns / width()) - 1;
         } else {
-            col = e->pos().x() * d->m_numDayColumns / width();
+            col = e->position().x() * d->m_numDayColumns / width();
         }
 
         const int pos = row < 1 ? -1 : (d->m_numDayColumns * (row - 1)) + col;
@@ -611,7 +610,7 @@ void KDateTable::mousePressEvent(QMouseEvent *e)
         QMenu *menu = new QMenu();
         menu->addSection(locale().toString(d->m_date));
         Q_EMIT aboutToShowContextMenu(menu, clickedDate);
-        menu->popup(e->globalPos());
+        menu->popup(e->globalPosition().toPoint());
     }
 }
 
